@@ -17,10 +17,12 @@ try:
     from flask import request
 except ImportError:
     raise ImportError('You must install flask `pip install flask` to use this module')
+from flask_cors import CORS
 
 from mapshader import hello
 from mapshader.core import render_map
 from mapshader.core import render_geojson
+from mapshader.core import render_legend
 
 from mapshader.sources import get_services
 from mapshader.sources import MapSource
@@ -82,6 +84,11 @@ def flask_to_geojson(source: MapSource):
     return resp
 
 
+def flask_to_legend(source: MapSource):
+    resp = render_legend(source)
+    return resp
+
+
 def build_previewer(service: MapService):
     '''Helper function for creating a simple Bokeh figure with
     a WMTS Tile Source.
@@ -202,11 +209,14 @@ def index_page(services):
 
 def configure_app(app, user_source_filepath=None, contains=None):
 
+    CORS(app)
+
     view_func_creators = {
         'tile': flask_to_tile,
         'image': flask_to_image,
         'wms': flask_to_wms,
         'geojson': flask_to_geojson,
+        'legend': flask_to_legend,
     }
 
     services = []
@@ -221,6 +231,11 @@ def configure_app(app, user_source_filepath=None, contains=None):
                          service.name,
                          partial(view_func, source=service.source))
 
+        # add legend endpoint
+        app.add_url_rule(service.legend_url,
+                         service.legend_name,
+                         partial(view_func_creators['legend'], source=service.source))
+
         # add service page endpoint
         app.add_url_rule(service.service_page_url,
                          service.service_page_name,
